#!/bin/bash
# Multiple Post REST File Upload Example Script
# Oct-23-2020 j.clark@f5.com
# Based on Python script by Jason Rahm, F5
# https://devcentral.f5.com/s/articles/demystifying-icontrol-rest-part-5-transferring-files
 
# 1=debug on, 0=debug off
debug=0
 
# Each argument saved as a variable (these are simply for reference and can be removed)
ipaddress=$1
username=$2
password=$3
filename=$4
path=$5
destfilename=$6
[ $debug = 1 ] && echo "Address: $ipaddress User: $username Password: $password Local File: $filename Remote Path: $path Remote File: $destfilename"
 
# dd has a default block size of 512. It is specified manually here
# to allow tuning.
blocksize=512
# blocknumber * blocksize = total size of file segment
# Number of blocks to read at a time.
# 30720 results in ~15mb segments, getting almost as good performance
# as 30MB without getting near the 33MB ceiling of 12.1.+.
blocknumber=30720
# 61440 results in ~30MB segment, the largest size for 12.1+.
#blocknumber=61440
# 2048 results in 1MB segments the same size as BIG-IQ uses.
# Required for 12.0.x.
#blocknumber=2048
 
# Length of the file, in bytes
filelength=$(stat -c%s $filename)
[ $debug = 1 ] && echo "File length is: $filelength"
# Location/name of temporary file to hold the chunk
tempfile=/tmp/tempfile
# Initialize at Offset 0 so read starts at the 1st byte
offset=0
# Initialize end to -1 for single POST case
end=-1
# Initialize Connection type at Keep-Alive
connection="Keep-Alive"
 
echo "Uploading $4 to $1."
 
# Begin loop to iterate through the file and upload in chunks
while true
do
        [ $debug = 1 ] && echo "Starting iteration with offset of $offset"
        # Copy portion of file into tempfile as a chunk
        /bin/dd status=none if=$filename of=$tempfile count=$blocknumber skip=$offset bs=$blocksize     2> /dev/null
        [ $debug = 1 ] && echo "tempfile is $(stat -c%s $tempfile) bytes long"
        # Determine length of segment
        chunklength=$(stat -c%s $tempfile)
        # Determine if this segment completes the filelength
        if [ $(( $end + $chunklength + 1 )) -eq $filelength ]; then
                # This is last segment, set connection to Close
                [ $debug = 1 ] && echo "Final segment reached."
                connection=Close
        fi
        # Set start and end for this segment
        start=$(( $offset * $blocksize ))
        # end -1 to adjust to zero-based
        end=$(( $start + $chunklength - 1 ))
        [ $debug = 1 ] && echo "Content-Range: $start-$end/$filelength"
        [ $debug = 1 ] && echo "Connection: $connection"
        # Make the POST request, uploading the content chunk
        # BIG-IP REST JSON response piped to /dev/null
        curl -sku $username:$password https://$ipaddress$path$destfilename -X POST -H "Content-Type: application/octet-stream" -H "Content-Range: $start-$end/$filelength" -H "Expect: " -H "Connection: $connection" --data-binary @$tempfile > /dev/null
        # Print a period to indicate progress.
        printf "."
        # If this was final segment, end loop
        if [ $connection = "Close" ]; then
                [ $debug = 1 ] && echo "Breaking loop."
                # Print complete
                printf "done.\n"
                break
        fi
        # Increment the offset by the number of blocks read at a time
        offset=$(( $offset + $blocknumber ))
        [ $debug = 1 ] && echo "offset increased to $offset"
done
# Delete the tempfile
rm -f /tmp/tempfile
# Display md5sum for the local file to compare to remote file
printf "Calculating md5sum for $4 to compare to uploaded file:\n"
md5sum $4
